@@ -17,15 +17,19 @@
 
 import logging
 import mimetypes
-from flask import abort, request
+from pygments import highlight
+from pygments.lexers import get_lexer_for_filename
+from pygments.formatters import HtmlFormatter
+from flask import request, make_response
 from flask_wtf.csrf import generate_csrf
 from airflow.version import version
-from airflow_code_editor.commons import HTTP_404_NOT_FOUND
-from airflow_code_editor.tree import get_tree
+from airflow_code_editor.commons import HTTP_200_OK, HTTP_404_NOT_FOUND
+from airflow_code_editor.tree import get_tree, get_stat
 from airflow_code_editor.utils import (
+    DummyLexer,
+    error_message,
     get_plugin_boolean_config,
     get_plugin_int_config,
-    error_message,
     normalize_path,
     prepare_api_response,
 )
@@ -63,9 +67,7 @@ class AbstractCodeEditorView(object):
             logging.error(ex)
             return prepare_api_response(
                 path=normalize_path(path),
-                error_message="Error saving {path}: {message}".format(
-                    path=path, message=error_message(ex)
-                ),
+                error_message="Error saving {path}: {message}".format(path=path, message=error_message(ex)),
             )
 
     def _git_repo(self, path):
@@ -78,15 +80,14 @@ class AbstractCodeEditorView(object):
         "Get a file from GIT (invoked by the HTTP GET method)"
         try:
             # Download git blob - path = '<hash>/<name>'
-            path, attachment_filename = path.split('/', 1)
-        except:
+            path, attachment_filename = path.split("/", 1)
+        except Exception:
             # No attachment filename
             attachment_filename = None
-        response = execute_git_command(["cat-file", "-p", path])
+        response = execute_git_command(["cat-file", "-p", path]).prepare_git_response()
         if attachment_filename:
-            response.headers["Content-Disposition"] = (
-                'attachment; filename="{0}"'.format(attachment_filename)
-            )
+            content_disposition = 'attachment; filename="{0}"'.format(attachment_filename)
+            response.headers["Content-Disposition"] = content_disposition
             try:
                 content_type = mimetypes.guess_type(attachment_filename)[0]
                 if content_type:
@@ -97,8 +98,8 @@ class AbstractCodeEditorView(object):
 
     def _git_repo_post(self, path):
         "Execute a GIT command (invoked by the HTTP POST method)"
-        git_args = request.json.get('args', [])
-        return execute_git_command(git_args)
+        git_args = request.json.get("args", [])
+        return execute_git_command(git_args).prepare_git_response()
 
     def _load(self, path):
         "Send the contents of a file to the client"
@@ -114,7 +115,10 @@ class AbstractCodeEditorView(object):
                 return root_fs.path(path).send_file(as_attachment=True)
         except Exception as ex:
             logging.error(ex)
-            abort(HTTP_404_NOT_FOUND)
+            strerror = getattr(ex, "strerror", str(ex))
+            errno = getattr(ex, "errno", 0)
+            message = prepare_api_response(error_message=strerror, errno=errno)
+            return make_response(message, HTTP_404_NOT_FOUND)
 
     def _format(self):
         "Format code"
@@ -134,16 +138,51 @@ class AbstractCodeEditorView(object):
             return prepare_api_response(
                 error_message="black dependency is not installed: to install black `pip install black`"
             )
+        except Exception as ex:
+            logging.error(ex)
+            return prepare_api_response(error_message="Error formatting: {message}".format(message=error_message(ex)))
+
+    def _tree(self, path, args={}, method="GET"):
+        try:
+            if method == "HEAD":
+                stat = get_stat(path)
+                response = make_response("OK", HTTP_200_OK)
+                response.headers["X-Id"] = stat["id"]
+                response.headers["X-Leaf"] = "true" if stat["leaf"] else "false"
+                response.headers["X-Exists"] = "true" if stat["exists"] else "false"
+                return response
+            else:
+                return prepare_api_response(value=get_tree(path, args))
         except Exception as ex:
             logging.error(ex)
             return prepare_api_response(
-                error_message="Error formatting: {message}".format(
-                    message=error_message(ex)
-                )
+                value=[],
+                error_message="Error: {message}".format(message=error_message(ex)),
             )
 
-    def _tree(self, path, args={}):
-        return {'value': get_tree(path, args)}
+    def _search(self, args={}):
+        "File search"
+        query = args.get("query")
+        root_fs = RootFS()
+        result = []
+        for match in root_fs.search(query=query):
+            formatter = HtmlFormatter(
+                linenos=True,
+                cssclass="source",
+                nobackground=True,
+                linenostart=match["context_first_row"],
+                hl_lines=[match["row_number"] - match["context_first_row"] + 1],
+            )
+            try:
+                lexer = get_lexer_for_filename(match["path"])
+            except Exception:
+                lexer = DummyLexer()
+            try:
+                context = highlight(match["context"], lexer, formatter)
+            except Exception:
+                context = match["context"]
+            result.append({"row_number": match["row_number"], "context": context, "path": match["path"]})
+        return result
 
     def _ping(self):
-        return {'value': generate_csrf()}
+        return {"value": generate_csrf()}
