@@ -19,7 +19,13 @@ from flask import request
 import airflow
 from functools import wraps
 from airflow_code_editor.code_editor_view import AbstractCodeEditorView
-from airflow_code_editor.commons import ROUTE, MENU_CATEGORY, MENU_LABEL, JS_FILES
+from airflow_code_editor.commons import (
+    ROUTE,
+    MENU_CATEGORY,
+    MENU_LABEL,
+    JS_FILES,
+    VERSION,
+)
 
 __all__ = ["admin_view"]
 
@@ -70,15 +76,20 @@ try:
         def format(self, path=None):
             return self._load(path)
 
-        @expose("/tree", methods=["GET"])
+        @expose("/tree", methods=["GET", "HEAD"])
         @login_required
         def tree_base(self, path=None):
-            return self._tree(path, args=request.args)
+            return self._tree(path, args=request.args, method=request.method)
 
-        @expose("/tree/<path:path>", methods=["GET"])
+        @expose("/tree/<path:path>", methods=["GET", "HEAD"])
         @login_required
         def tree(self, path=None):
-            return self._tree(path, args=request.args)
+            return self._tree(path, args=request.args, method=request.method)
+
+        @expose("/search", methods=["GET"])
+        @login_required
+        def search(self):
+            return self._search(args=request.args)
 
         @expose("/ping", methods=["GET"])
         @login_required
@@ -90,6 +101,7 @@ try:
                 template + "_admin.html",
                 airflow_major_version=self.airflow_major_version,
                 js_files=JS_FILES,
+                version=VERSION,
                 *args,
                 **kargs
             )
