@@ -15,12 +15,19 @@
 #   limitations under the Licens
 
 import re
+import fs
 from datetime import datetime
 from typing import Any, Callable, Dict, List, NamedTuple, Optional
 from airflow_code_editor.commons import (
     Args,
     TreeFunc,
     TreeOutput,
+    ICON_HOME,
+    ICON_GIT,
+    ICON_TAGS,
+    FOLDER_ICON,
+    ICON_LOCAL_BRANCHES,
+    ICON_REMOTE_BRANCHES,
 )
 from airflow_code_editor.utils import (
     always,
@@ -33,7 +40,7 @@ from airflow_code_editor.git import (
 )
 from airflow_code_editor.fs import RootFS
 
-__all__ = ['get_tree']
+__all__ = ['get_tree', 'get_stat']
 
 
 class NodeDef(NamedTuple):
@@ -72,9 +79,7 @@ def get_root_node(path: Optional[str], args: Args) -> TreeOutput:
         if id_ is None or not node.condition():
             continue
         # Add the node
-        result.append(
-            {'id': id_, 'label': node.label, 'leaf': node.leaf, 'icon': node.icon}
-        )
+        result.append({'id': id_, 'label': node.label, 'leaf': node.leaf, 'icon': node.icon})
         # If the node is files, add the mount points
         if id_ == 'files':
             mount_points = read_mount_points_config()
@@ -84,19 +89,21 @@ def get_root_node(path: Optional[str], args: Args) -> TreeOutput:
                     {
                         'id': 'files/~' + mount,
                         'label': mount,
-                        'icon': 'fa-folder',
+                        'icon': FOLDER_ICON,
                         'leaf': False,
                     }
                 )
     return result
 
 
-@node(id='files', label='Files', leaf=False, icon='fa-home')
+@node(id='files', label='Files', leaf=False, icon=ICON_HOME)
 def get_files_node(path: Optional[str], args: Args) -> TreeOutput:
     "Get tree files node"
     result = []
-    long_ = 'long' in args
-    for item in RootFS().path(path).iterdir():
+    long_ = 'long' in args  # long format
+    all_ = 'all' in args  # do not ignore entries
+
+    for item in RootFS().path(path).iterdir(show_ignored_entries=all_):
         s = item.stat()
         leaf = not item.is_dir()
         if long_:  # Long format
@@ -107,9 +114,7 @@ def get_files_node(path: Optional[str], args: Args) -> TreeOutput:
                     'leaf': leaf,
                     'size': size,
                     'mode': s.st_mode,
-                    'mtime': datetime.fromtimestamp(int(s.st_mtime)).isoformat()
-                    if s.st_mtime
-                    else None,
+                    'mtime': datetime.fromtimestamp(int(s.st_mtime)).isoformat() if s.st_mtime else None,
                 }
             )
         else:  # Short format
@@ -121,10 +126,10 @@ def git_command_output(*args: str) -> List[str]:
     "Execute a git command and return the output as a list of string"
     r = execute_git_command(list(args))
     # Check exit code
-    if r.headers.get('X-Git-Return-Code') != '0':
+    if r.returncode != 0:
         return []
     # Return stdout lines
-    return r.data.decode('utf-8').split('\n')
+    return r.stdout.split('\n')
 
 
 def prepare_git_output(line: str, icon: str) -> Dict[str, Any]:
@@ -146,27 +151,27 @@ def prepare_ls_tree_output(line: str) -> Dict[str, Any]:
     }
 
 
-@node(id='git', label='Git Workspace', icon='fa-briefcase', condition=git_enabled)
+@node(id='git', label='Git Workspace', icon=ICON_GIT, condition=git_enabled)
 def get_git_node(path: Optional[str], args: Args) -> TreeOutput:
     "List the contents of a git tree object"
     output = git_command_output('ls-tree', '-l', path or 'HEAD')
     return [prepare_ls_tree_output(line) for line in output if line]
 
 
-@node(id='tags', label='Tags', leaf=False, icon='fa-tags', condition=git_enabled)
+@node(id='tags', label='Tags', leaf=False, icon=ICON_TAGS, condition=git_enabled)
 def get_tags_node(path: Optional[str], args: Args) -> TreeOutput:
     "Get tree tags node"
     if path:
         return get_git_node(path, args)
     output = git_command_output('tag')
-    return [prepare_git_output(line, 'fa-tags') for line in output if line]
+    return [prepare_git_output(line, ICON_TAGS) for line in output if line]
 
 
 @node(
     id='local-branches',
     label='Local Branches',
     leaf=False,
-    icon='fa-code-fork',
+    icon=ICON_LOCAL_BRANCHES,
     condition=git_enabled,
 )
 def get_local_branches_node(path: Optional[str], args: Args) -> TreeOutput:
@@ -174,14 +179,14 @@ def get_local_branches_node(path: Optional[str], args: Args) -> TreeOutput:
     if path:
         return get_git_node(path, args)
     output = git_command_output('branch')
-    return [prepare_git_output(line, 'fa-code-fork') for line in output if line]
+    return [prepare_git_output(line, ICON_LOCAL_BRANCHES) for line in output if line]
 
 
 @node(
     id='remote-branches',
     label='Remote Branches',
     leaf=False,
-    icon='fa-globe',
+    icon=ICON_REMOTE_BRANCHES,
     condition=git_enabled,
 )
 def get_remote_branches_node(path: Optional[str], args: Args) -> TreeOutput:
@@ -189,7 +194,7 @@ def get_remote_branches_node(path: Optional[str], args: Args) -> TreeOutput:
     if path:
         return get_git_node(path, args)
     output = git_command_output('branch', '--remotes')
-    return [prepare_git_output(line, 'fa-globe') for line in output if line]
+    return [prepare_git_output(line, ICON_REMOTE_BRANCHES) for line in output if line]
 
 
 def get_tree(path: Optional[str] = None, args: Args = {}) -> TreeOutput:
@@ -208,3 +213,17 @@ def get_tree(path: Optional[str] = None, args: Args = {}) -> TreeOutput:
         return []
     # Execute node function
     return TREE_NODES[root].get_children(path_argv, args)
+
+
+def get_stat(path: Optional[str] = None, args: Args = {}) -> TreeOutput:
+    "Get stat for the given path"
+    try:
+        if not path:
+            return {'id': path, 'leaf': False, 'exists': True}
+        path = normalize_path(path)
+        get_tree(path, args)
+        return {'id': path, 'leaf': False, 'exists': True}
+    except fs.errors.DirectoryExpected:
+        return {'id': path, 'leaf': True, 'exists': True}
+    except fs.errors.ResourceNotFound:
+        return {'id': path, 'leaf': None, 'exists': False}
